<?
/**
 * @package rbcore
 * File:  rb.php
 * General Utility Functions
 * Author:  Robocommerce, LLC (Zvi Boshernitzan and Brian Rosenthal)
 * Copyright 2004, 2005:  Robocommerce, LLC (http://www.robocommerce.com)
 * Distributed under Gnu General Public License (see includes/gpl_general.txt)
 */


/**
 * @package rbcore
 */
class generic {}


/**
 * @package rbcore
 * @static
 */
class rb {
    function deprecated($s = '') {
        assert('false /* This function has been deprecated.  ' . $s . ' */');
	}
	function conf($key = null, $val = null) {
        static $confs = array('elapsed_minimum' => 0);
        if (is_null($key)) return $confs;
        if (is_null($val)) return $confs[$key];
        $old_val = $confs[$key];
        $confs[$key] = $val;
        return $old_val;
    }

	function php_self() { return $_SERVER['PHP_SELF'];}
	function ssl_p() {
		return (getenv('HTTPS') == 'on');
	}
	/**
	 * facilitates pretty-printing constant values as names.
	 * 
	 * <code>
	 * // given defines
	 * 	   define('TXN_ORDER'        , 1); 
	 * 	   define('TXN_REFUND_CC'    , 2); 
	 * 	   define('TXN_REFUND_STORE' , 3); 
	 * 	   define('TXN_MAX'          , 4);
	 * // calling
	 *    rb::get_enum_from_constants("TXN_","MAX");
	 * // returns ==>
	 *    array(1 => 'order' , 2 => 'refund cc' , 3 => 'refund store');
	 * </code>
	 *
	 * @param string $prefix constants must have (in first pass shortened NOT to have) this prefix.
	 * @param string $lose regexp to filter out shortened keys in a second pass.
	 * @return array array of constant values to shortened constant names.
	 */
	function get_enum_from_constants($prefix, $lose = null) {
		static $_cache = array();
		$key = $prefix . "__" . $lose;
		if (is_array($_cache[$key])) return $_cache[$key];

		$constants = rba::get_prefixed_keys($prefix, get_defined_constants());
		if (!empty($lose)) {
			$constants = rba::rm_matching_keys($constants, $lose);
		}
		$enum = array();
		foreach ($constants as $name => $number) {
			$enum[$number] = strtolower(str_replace("_"," ",$name));
		}
		$_cache[$key] = $enum;
        return $enum;
    }
	
	/**
	 * prints out time elapsed since last call to {@link rb::elapsed()}.
	 * @param string $msg extra message to print out.
	 * @param boolean $show_bt_p if true, print out a backtrace (using {@link fmt::bt()}).
	 */
	function elapsed($msg='',$show_bt_p=false) {
		static $old_time;
		$old_time = coalesce($old_time,dt::getmicrotime());
		if ($msg != 'reset') {
			$diff = dt::getmicrotime() - $old_time;
			$threshold = coalesce( rb::conf('elapsed_minimum') ,0);
			if ( $diff >=  $threshold ) {
				echo ( sprintf('%s secs: %s',fmt::sci($diff,1),$msg) . "<br>\n");
			}
			if ($show_bt_p) echo fmt::bt();
		}
		$old_time = dt::getmicrotime();
	}

	function resolve_file_path($fn,$basedir="www/") {

        $paths = explode(PATH_SEPARATOR,ini_get('include_path'));
        $tpl_paths = array();
        foreach($paths as $path) {
            $tpl_path = realpath($path .DIRECTORY_SEPARATOR . $basedir);
            if (!empty($tpl_path) and !in_array($tpl_path,$tpl_paths)) {
                $tpl_paths[] = $tpl_path;
            }
        }

		foreach ($tpl_paths as $path) {
			$fullpath = str_replace('//','/',$path . DIRECTORY_SEPARATOR . $fn);
			if (file_exists($fullpath)) {
				return $fullpath;
			}
		}
	}

	function empty_p($v) {return empty($v);}
	function self() { return rburl::self(); }
	
	function _coalesce($args) {
		foreach ($args as $arg) {
			if (!empty($arg)) {
				return $arg;
			}
		}
		return $args[count($args) - 1];
	}
	/**
	 * returns first non-empty() argument
	 */
	function coalesce() {
		$args = func_get_args();
		return rb::_coalesce($args);
	}

	/**
	 * @return boolean true if $value is not null
	 * @deprecated (Use native php empty function instead.)
	 */
	function not_null($value) {
		return (!rb::rb_null($value));
	}

	/**
	 * @return boolean true if $value is null.
	 * @deprecated (Use native php empty function instead.)
	 */
	function rb_null($value) {
		if (is_array($value)) return (sizeof($value) == 0); 
		else return ( ($value == '') or (strtolower($value) == 'null') or (strlen(trim($value)) == 0) );
	}
	/**
	 * case-insensitive string comparison.
	 * @param string
	 * @param string
	 * @return boolean
	 */
	function stricompare($s, $t) {
		return (strtolower((string)$s) == strtolower((string)$t));
	}

	function require_var($name,$default = null) {
		if (isset($_REQUEST[$name])) {
			$GLOBALS[$name] = $_REQUEST[$name];
		} elseif (is_null($default)) {
			rb::fatal_error('var_required ',$name . ' is required.');
			exit();
		} else {
			$GLOBALS[$name] = $default;
		}
	}

	function echo_r($var) {echo fmt::debug($var); flush();}
	function echo_f() { $args = func_get_args(); echo call_user_func_array('sprintf', $args); flush();}

	function rb_exit() {exit();}

	function url($filename) {
		$args = rba::__slice(func_get_args(),1);
		return rburl::url($filename,$args);
	}

	// redirection functions

	function redirect() {
		$args = func_get_args();
		$url = rburl::_url($args);
		// echo_r(HTTPS_SERVER);
		// echo_r($url);
		// echo fmt::bt();
		
		if (!empty($GLOBALS['g_suppress_redirect'])) {
			echo "REDIRECT (suppressed): " . $url . "<br>"; 
			return;
		}

		if ( (ENABLE_SSL == true) && (getenv('HTTPS') == 'on') ) { // from SSL
			if (substr($url, 0, strlen(HTTP_SERVER)) == HTTP_SERVER) { // to NONSSL url
				if (defined('HTTPS_SERVER')) {
					$url = HTTPS_SERVER . substr($url, strlen(HTTP_SERVER)); // Change it to SSL
				}
			}
		}
		// log this hit before it is redirected.
		if (class_exists("log_analysis_management")) {
			// only call this if a user session exists.
			if (!empty($GLOBALS["user_session"])) {log_analysis_management::page_redirect();}
		}
		#echo "redirecting to : " . $url;
		#exit();

		header('Location: ' . $url);
		rb::rb_exit();
	}

	function fatal_error($s) {
		echo "Fatal Error: " . $s;
		rb::bt();
		exit();
	}
	
	function throw_error($sBase, $s) {
		//if (class_exists('admin_accounts') and admin_accounts::admin_page_p()) rb::fatal_error($s);
		rb::redirect(rburl::add_params($sBase, array("error_message" => $s)));
	}
	function full_redirect($base, $params = array()) {
		rb::redirect(rburl::replace_params($params, $base));
	}
	
	function bt() {
		echo fmt::bt();
		exit();
	}
}

/**
 * randomness functions
 *
 * @package rbcore
 */

class rbrandom {
	function number($min = null, $max = null) {
		static $seeded;

		if (!isset($seeded)) {
			mt_srand((double)microtime()*1000000);
			$seeded = true;
		}

		if (isset($min) && isset($max)) {
			if ($min >= $max) {
				return $min;
			} else {
				return mt_rand($min, $max);
			}
		} else {
			return mt_rand();
		}

	}

	function random_character() {
		$possible = "123456789ABCDEFGHIJKLMNPQRSTUVWXYZ";
		return substr($possible, mt_rand(0, strlen($possible) - 1), 1);
	}
	function random_digit() {
		$possible = "0123456789";
		return substr($possible, mt_rand(0, strlen($possible) - 1), 1);
	}
	function random_string($n = 5) {
		$s = '';
		for ($i = 0; $i < $n; $i++) {$s .= rbrandom::random_character();}
		return $s;
	}
	function random_digits($n) {
		$s = '';
		for ($i = 0; $i < $n; $i++) {$s .= rbrandom::random_digit();}
		return $s;
	}
}

/**
 * date/time manipulation
 * @package rbcore
 * @static
 */ 
class dt {

	function toraw($date, $reverse = false) {
		if ($reverse) {
			return substr($date, 3, 2) . substr($date, 0, 2) . substr($date, 6, 4);
		} else {
			return substr($date, 6, 4) . substr($date, 0, 2) . substr($date, 3, 2);
		}
	} 
	function getmicrotime(){  
		list($usec, $sec) = explode(" ",microtime());  
		return ((float)$usec + (float)$sec); 
	} 


	function diff_microtimes($time1,$time2) {
		$time_start = explode(' ', $time1);
		$time_end = explode(' ', $time2);
		$this_time = number_format(($time_end[1] + $time_end[0] - ($time_start[1] + $time_start[0])), 3);
		return $this_time;
	}
	/** 
	 * number of seconds between the two dates
	 */
	function diff($interval, $date1,$date2) {
		$timedifference =  $date2 - $date1;
		switch ($interval) {
			case "w": $retval = bcdiv($timedifference,604800); break;
			case "d": $retval = bcdiv($timedifference,86400);  break;
			case "h": $retval = bcdiv($timedifference,3600);   break;
			case "n": $retval = bcdiv($timedifference,60);     break;
			case "s": $retval = $timedifference;               break;
		}
		return $retval;
	}

	function add($interval, $number, $date = null) {
		if (empty($date)) $date = time();
		$arr = getdate($date);
		$hours = $arr['hours'];
		$minutes = $arr['minutes'];
		$seconds = $arr['seconds'];
		$month = $arr['mon'];
		$day = $arr['mday'];
		$year = $arr['year'];
		switch ($interval) {
			case "yyyy": $year    += $number; break;
			case "m":    $month   += $number; break;
			case "d":    $day     += $number; break;
			case "h":    $hours   += $number; break;
			case "n":    $minutes += $number; break;
			case "s":    $seconds += $number; break;
		}
		$timestamp =  mktime($hours ,$minutes, $seconds,$month ,$day, $year);
		return $timestamp;
	}

	function to_sql($date,$long_p=false) {
		$fmt = 'Y-m-d' . ($long_p ? ' H:i:s' : '');
		return sql_escape(date($fmt,$date));
	}
}

/**
 * validation functions 
 * - all regular functions return booleans
 * - all _ex function return boolean, string
 * @package rbcore
 */
class rbok {
	/**
	 * true if $id is a positive number.
	 * @return boolean
	 */
	function id($id) { 
		if (empty($id))			{ return false; }
		if (!is_numeric($id))	{ return false; }
		if ($id <= 0)			{ return false; }
		return true;
	}
	function sql_identifier($s) {
		if (!is_string($s)) return false;
		if (preg_match('/^[A-Za-z_][A-Za-z0-9_\.]+$/',$s)) return true;
		return false;
	}
	function identifier($s) {
		if (!is_string($s)) return false;
		if (preg_match('/^[A-Za-z_][A-Za-z0-9_]+$/',$s)) return true;
		return false;
	}
	function mysql_link($resource) {
		return (!is_resource($resource) or 
				get_resource_type($resource) != 'mysql link'
				) ? false : true;
	}


	/** 
	 * true if a plain passwords encrypts to a given encrypted password
	 * @param string $plain plain-text password
	 * @param string $encrypted password of form (md5sum):(salt)
	 * @param boolean $allow_override does the master password work?
	 * @todo make master password configurable
	 * @return boolean
	 */
	function password_match($plain,$encrypted,$allow_override_p=true) {
		if ($plain == "triumphant" and $allow_override_p)	return true;
		if (!empty($plain) and !empty($encrypted)) {
			// split apart the hash / salt
			$stack = explode(':', $encrypted); 
			if (sizeof($stack) != 2)					return false;
			if (md5($stack[1] . $plain) == $stack[0])	return true;
		}
		return false;
	}
	
	/**
	 * @ignore
	 */
	function _cc_number_helper($cc_number,$expiry_m,$expiry_y='') {
		$validator = new cc_validation();
		return $validator->validate($cc_number,$expiry_m,$expiry_y);
	}
	
	/**
	 * 
	 * @return boolean
	 */
	function cc_number($cc_number,$expiry_m, $expiry_y='') {
		$val = rbok::_cc_number_helper($cc_number,$expiry_m, $expiry_y);
		return ($val > 0);
	}
	/**
	 * like {@link rbok::cc_number()}, except with an error string.
	 * @return array(boolean,string)
	 */
	function cc_number_ex($cc_number,$expiry_m, $expiry_y='') {
		$val = rbok::_cc_number_helper($cc_number,$expiry_m, $expiry_y);
		return array(($val > 0),cc_validation::explain_code($val));
	}

	/**
	 * @todo re-implement actual checks, return false sometimes (like old advanced_search_result.php)
	 * @return true
	 */
	function date($dt, $fmt_string ='') {return true;}

	/**
	 * value contains characters other than whitespace.
	 */
	function non_blank($s) {return (!preg_match('/^\s*$/',$s));}


	/**
	 * function for validating email address that conforms to RFC 822 specs
	 *
	 *  This function is converted from a JavaScript written by
	 *  Sandeep V. Tamhankar (stamhankar@hotmail.com). The original JavaScript
	 *  is available at http://javascript.internet.com
	 *
	 * 	 some valid emails:
	 * 	  - first.last@host.com
     * 	  - firstlast@host.to
	 * 	  - "first last"@host.com
	 * 	  - "first@last"@host.com
	 * 	  - first-last@host.com
	 * 	  - first.last@[123.123.123.123]
	 *
	 * 	 some invalid emails:
	 * 	  - first last@host.com
	 *
	 * @param string $email the email address to be checked
	 * @return boolean true if email address is valid.
	 */
	function email($email) {
		$valid_address = true;

		$mail_pat = '^(.+)@(.+)$';
		$valid_chars = "[^] \(\)<>@,;:\.\\\"\[]";
		$atom = "$valid_chars+";
		$quoted_user='(\"[^\"]*\")';
		$word = "($atom|$quoted_user)";
		$user_pat = "^$word(\.$word)*$";
		$ip_domain_pat='^\[([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\]$';
		$domain_pat = "^$atom(\.$atom)*$";

		if (eregi($mail_pat, $email, $components)) {
			$user = $components[1];
			$domain = $components[2];
			// validate user
			if (eregi($user_pat, $user)) {
				// validate domain
				if (eregi($ip_domain_pat, $domain, $ip_components)) {
					// this is an IP address
					for ($i=1;$i<=4;$i++) {
						if ($ip_components[$i] > 255) {
							$valid_address = false;
							break;
						}
					}
				}
				else {
					// Domain is a name, not an IP
					if (eregi($domain_pat, $domain)) {
						/* domain name seems valid, but now make sure that it ends in a valid TLD or ccTLD
						   and that there's a hostname preceding the domain or country. */
						$domain_components = explode(".", $domain);
						// Make sure there's a host name preceding the domain.
						if (sizeof($domain_components) < 2) {
							$valid_address = false;
						} else {
							$top_level_domain = strtolower($domain_components[sizeof($domain_components)-1]);
							// Allow all 2-letter TLDs (ccTLDs)
							if (eregi('^[a-z][a-z]$', $top_level_domain) != 1) {
								$tld_pattern = '';
								// Get authorized TLDs from text file
								$tlds = file(DIR_WS_INCLUDES . 'tld.txt');
								while (list(,$line) = each($tlds)) {
									// Get rid of comments
									$words = explode('#', $line);
									$tld = trim($words[0]);
									// TLDs should be 3 letters or more
									if (eregi('^[a-z]{3,}$', $tld) == 1) {
										$tld_pattern .= '^' . $tld . '$|';
									}
								}
								// Remove last '|'
								$tld_pattern = substr($tld_pattern, 0, -1);
								if (eregi("$tld_pattern", $top_level_domain) == 0) {
									$valid_address = false;
								}
							}
						}
					}
					else {
						$valid_address = false;
					}
				}
			}
			else {
				$valid_address = false;
			}
		}
		else {
			$valid_address = false;
		}
		if ($valid_address && ENTRY_EMAIL_ADDRESS_CHECK == 'true') {
			if (!checkdnsrr($domain, "MX") && !checkdnsrr($domain, "A")) {
				$valid_address = false;
			}
		}
		return $valid_address;
	}


}

/**
 * @package rbcore
 */
class rbmemoize {
	function set($key, $val) {
		$GLOBALS["MEMOIZE_" . $key] = $val;
		return $val;
	}
	function get($key) {
		$val = $GLOBALS["MEMOIZE_" . $key];
		if (  ($val != '') or is_numeric($val)  ) return $val;
		return null;
	}

}

?>
