<?
/**
 * @package rbcore
 * File:  rba.php
 * Array manipulation functions
 * Author:  Robocommerce, LLC (Zvi Boshernitzan and Brian Rosenthal)
 * Copyright 2004, 2005:  Robocommerce, LLC (http://www.robocommerce.com)
 * Distributed under Gnu General Public License (see includes/gpl_general.txt)
 * Version 1.0
 */

/**
 * array manipulation
 * @package rbcore
 */
class rba {

	function keys_not_empty($arr, $required) {
		foreach ($required as $key) {
			if (empty($arr[$key])) { return false;}
		}	
		return true;
	}
	function keys_exist($arr, $required) {
		foreach ($required as $key => $val) {
			if (!isset($arr[$val])) { return false;}
		}
		return true;
	}

	/**
	 * allow php4 to index expressions that return arrays.
	 */
	function get($arr,$key) {return $arr[$key];}


	function range($start, $end = null) {
		if (empty($end)) {$end = $start; $start = 0;}
		for ($i = $start; $i <= $end; $i++) $arr[] = $i;
		return coalesce($arr,array());
	}

	/**
	 *  aliases for get_object_vars
	 */
	function from_obj($obj)    {return get_object_vars($obj);}
	function from_object($obj) {return get_object_vars($obj);}
	
	/**
	 *  convert anything to an array.
	 */
	function to_array($mixed) {
		if (is_object($mixed)) return get_object_vars($mixed);
		if (is_scalar($mixed)) return array($mixed);
		if (is_array($mixed)) return $mixed;
		if (is_null($mixed)) { return array();}
		assert('false /* should not get here */');
	}

	/**
	 *  converts an array into an object of type "class_name"
	 */
	function to_object($arr, $classorobj = "generic") {
		return rba::to_obj($arr, $classorobj);
	}
	function to_obj($arr, $classorobj = "generic") {
		if (empty($arr)) return null;
		if (is_object($classorobj)) {
			$obj = $classorobj;
		} elseif (class_exists($classorobj)) {
			$obj = new $classorobj;
		} else  {
			echo "ERROR:  to_obj called on class that does not exist: " . $classorobj . ".";
			$obj = new $classorobj;
		}
		rba::set_object_vars($obj,$arr);
		return $obj;
	}
	function set_object_vars(&$obj, $vars) {
		foreach ($vars as $key => $value) {$obj->$key = $value;}
		return $obj;
	}
	
	/**
	 *  copy $arr, remove $keys and empty values if $rm_empty_p, and return copy.
	 *
	 *    - $keys is an array whose VALUES are the keys to remove.
	 */
	function rmkeys($arr,$keys,$rm_empty_p=false) {
		assert('is_array($arr)');
		$return_arr = $arr;
		foreach($keys as $key) {unset($return_arr[$key]);}
		if ($rm_empty_p) $return_arr = array_filter($return_arr,'strlen');
		return $return_arr;
	}
	function rm_matching_keys($arr,$re) {
		$re = '/' . trim($re,'/') . '/';
		foreach (array_keys($arr) as $key) {
			if (preg_match($re, $key)) unset($arr[$key]);
		}
		return $arr;
	}

	/**
	 *  return copy of $arr with some keys' values converted to 0 or 1
	 *
	 *  processes all keys ending with '_p' or passed in as extra args.
	 *
	 *  usage: 
	 *   - rb::bools_to_ints($_REQUEST);
	 *   - rb::bools_to_ints($_REQUEST,'status','is_poorly_formatted');
	 */
	function bools_to_ints($arr) {
		foreach ($arr as $key => $val) {
			if (preg_match('/_p$/',$key)) {
				$arr[$key] = (empty($val) ? 0 : 1 );
			}
		}
		$extras = coalesce(array_splice(func_get_args(),1),array());
		foreach($extras as $key) {
			$arr[$key] = (empty($arr[$key]) ? 0 : 1 );
		}
		return $arr;
	}

	/**
	 *  compact way to merge, add keys, or remove keys of arrays
	 *  
	 *  @param $args... arrays, (k,v pairs), or '-' to unset a key or an array of keys
	 *
	 *  usage: 
	 *   - rba::_($arr1,$arr2,'newkey','newval','-','badvar');
	 *   - rba::_($arr1,$arr2,'-',array('badvar1','badvar2'));
	 *   - rba::_($arr1,'-',array_keys($arr3));
	 */
	function _() {$args = func_get_args(); return rba::__($args);}		
	function __slice($args,$n=0) {
		if ($n > 0) $args = array_slice($args,$n);
		return rba::__($args);
	}
	function slice($args,$n=0) {
		if ($n > 0) $args = array_slice($args,$n);
		return $args;
	}

	/**
	 * @ignore
	 */
	function __($args) {
		//echo_r("nargs: " . count($args));
		$params = array();
		$next = 'any';
		$next_key = null;
		foreach($args as $arg) {
			if ($next == 'sub') {
				if (!is_array($arg)) $arg = array($arg);
				$params = rba::rmkeys($params,$arg);
				$next = 'any';
			} elseif ($next == 'any') {
				if ($arg == '-')		{$next='sub';}
				elseif (is_string($arg)){$next='val';$next_key=$arg;}
				elseif (is_array($arg))	{$params = array_merge($params,$arg);}
				else {continue;}
			} elseif ($next == 'val') {
				$params[$next_key] = $arg;
				$next_key = null;
				$next = 'any';
			} else {}
		}
		return $params;
	}		
	
	/** 
	 * returns subset of the keys in a dictionary as an array.
	 *
	 *  - the dictionary can be an object or an array
	 *  - result does not include keys not set in first argument.
	 *    (same behavior as an extract / compact pair).
	 *
	 * usage: 
	 *  - $new_array = rba::subset($array,'key1','key2');
	 *  - extract(rba::subset($_REQUEST,'var1','var43'));
	 *
	 * @param array $___arr first argument is an array
	 * @param string $argn... keys to allow in the subset
	 * @return array subset of the keys in the dictionary
	 */
	function subset($___arr) {
		if (is_object($___arr)) $___arr = get_object_vars($___arr);
		if (is_array($___arr)) {
			extract($___arr);
			$___args = func_get_args();
			return compact(array_slice($___args,1));
		}
		rb::fatal_error("First argument must be object or array.");
	}
	/**
	 * is the first argument's value found as one of the subsequent arguments?
	 */
	function in($needle) {
		$args = func_get_args();
		$haystack = array_slice($args,1);
		return in_array($needle,$haystack);
	}
	function validate_values($test_value) {
		$ok_values = rba::slice(func_get_args(),1);
		if (!in_array($test_value,$ok_values)) {
			echo_r(compact('test_value','ok_values'));
			rb::bt();
		}
	}
	function validate_keys($params,$ok_keys) {
		$bad_keys = array_diff(array_keys($params),$ok_keys);
		if (count($bad_keys) > 0) {echo_r(compact('bad_keys','ok_keys')); rb::bt();}
	}

	function merge_defaults($params,$default_params,$validate_keys_p = true) {
		$default_keys = array_keys($default_params);
		if ($validate_keys_p) rba::validate_keys($params,$default_keys);	
		$clean_params = rba::subset($params,$default_keys);
		return array_merge($default_params,$clean_params);
	}

	/**
	 * @param array $keys keys of new array
	 * @param array $values values of new array
	 * @return array|false with keys,value being $keys,$values respectively, errors false if number of keys and values is not equal.
	 */
	function combine ( &$keys, &$values ) {
		if ( count($keys) == 0 or count($keys) != count($values) ) return false;
		$result = array ();
		for ( $i = 0; $i < count($keys); $i++ ) {
			$result[$keys[$i]] = $values[$i];
		}
		return $result;
	}
	function combine_assoc(&$keys,&$values) {
		$result = array();
		if ( !count($values) || ( count($values[0]) !== count($keys) ) ) return false;
		for ( $i = 0; $i < count ( $values ); $i++ ) {
			$result [$i] = rba::combine ( $keys, $values[$i] );
		}
		return $result;
	}
	/** 
	 * extracts keys/values from array whose keys begin with prefix, stripping the prefix from the keys.
	 * @return array new array whose keys had (but no longer have) the prefix.
	 */
	function get_prefixed_keys($prefix,$arr = array() ) {
		if (is_object($arr)) $arr = rba::to_array($arr);
		assert(' !is_array($prefix) and is_array($arr) '
				. '/* you should call get_prefixed_keys($prefix,$arr) in that order */');
		if (empty($arr)) $arr = $_REQUEST;
		$prefix_len = strlen($prefix);
		$result = array();
		foreach ($arr as $key => $val) {
			if (strpos($key,$prefix) === 0) {
				$new_key = substr($key,$prefix_len);
				$result[$new_key] = $val;
			}
		}
		return $result;
	}

	function add_prefix($prefix,$arr = array()) {
		if (is_object($arr)) $arr = rba::to_array($arr);
		$result = array();
		foreach($arr as $key => $val) {
			$result[$prefix . $key] = $val;
		}
		return $result;
	}

	/**
	 * is the string a key in the array (lowercase comparison)
	 */
	function striinarray($s,$arr) {
		foreach($arr as $k => $v) {
			if (rb::stricompare($s,$k)) return true;
		}
		return false;
	}
	/** 
	 *  calls a function on each of a given array's keys, and sets
	 *  another key to the result.
	 *
	 */
	function map_keys($arr,$key_in,$key_out,$xform_function) {
		$result = array();
		foreach ($arr as $row) {
			$row[$key_out] = call_user_function($xform_function,$row[$key_in]);
			$result[] = $row;
		}
		return $result;
	}

}


?>
